/* EOPL syntax for source language */
/* Stephanie Weirich */

%token LAMBDA LPAREN RPAREN PLUS TIMES MINUS LET COMMA
%token <string>ID
%token <int> NUMBER
%token TRUE FALSE 
%token IF EQUAL AND OR NOT THEN ELSE IN LETREC SET
%token LT GT ZERO ADD1 SUB1

%start main
%type <Absyn.exp> main

%%

main: 
          exp { $1 }
;

exp: 
          ID { Absyn.Var ($1) }
        | prim LPAREN cexps RPAREN { Absyn.Prim($1,$3) }
		  | NUMBER { Absyn.LitInt ($1) }
		  | TRUE   { Absyn.LitBool (true) }
		  | FALSE  { Absyn.LitBool (false) }
		  | IF exp THEN exp ELSE exp 
				     { Absyn.If ($2,$4,$6) }
		  | LET binds IN exp
				     { Absyn.Let (fst $2,snd $2, $4) }
		  | LAMBDA LPAREN none_or_ids RPAREN exp 
				     { Absyn.Lambda ($3,$5) }
		  | LPAREN exp exps RPAREN 
				     { Absyn.App ($2,$3) }
		  | LETREC recbinds IN exp 
				     { let (a,b,c) = $2 in Absyn.Letrec(a,b,c,$4) }
;

none_or_ids :  { [] } 
        | ids  { ($1) }
;

ids:   
          ID { [$1] }
		  | ID COMMA ids { ($1::$3) }
;

binds:   { [],[] }
	     | ID EQUAL exp binds { ($1::fst $4,$3::snd $4) }
; 

recbinds: { [],[],[] }
        | ID LPAREN ids RPAREN EQUAL exp recbinds
			 { let (procnames, idss, bodies) = $7 in 
				  ($1::procnames, $3::idss, $6::bodies) }
;


prim:     PLUS  { Absyn.Plus  }
        | TIMES { Absyn.Times }
        | MINUS { Absyn.Minus }
		  | EQUAL { Absyn.Equal }
		  | AND   { Absyn.And   }
		  | OR    { Absyn.Or    }
		  | NOT   { Absyn.Not   }
		  | ADD1  { Absyn.Add1  }
		  | SUB1  { Absyn.Sub1  }
		  | LT    { Absyn.LT    }
		  | GT    { Absyn.GT    }
		  | ZERO  { Absyn.Zero  }

;

exps:     { [] } 
        | exp exps  { ($1 :: $2) }
;

cexps:    exp  { [$1] } 
        | exp COMMA exps  { ($1 :: $3) }
;
%%
